﻿using System.Collections.ObjectModel;
using Aurora.Models.Media;
using Aurora.Services;
using Aurora.Services.PlayerService;
using Xamarin.Forms;

namespace Aurora.Design.Views.Songs
{
    public class SongsViewModel : BaseViewModel
    {
        #region Fields
        private ObservableCollection<BaseMedia> _songsList;
        private BaseMedia _selectedSong;
        private PlayerService _player;

        #endregion Fields

        #region Constructor
        public SongsViewModel()
        {
            _player = PlayerService.Instance;
            _songsList = new ObservableCollection<BaseMedia>();
            PlayCommand = new Command(OnPlayExecute, CanPlayExecute);

            _player.PlaybackStateChanged += OnPlaybackStateChanged;

            Initialize();
        }

        #endregion Constructor

        #region Properties
        public ObservableCollection<BaseMedia> SongsList
        {
            get { return _songsList; }
            set { SetProperty(ref _songsList, value); }
        }

        public BaseMedia SelectedSong
        {
            get { return _selectedSong; }
            set { SetProperty(ref _selectedSong, value); }
        }

        public Command PlayCommand { get; private set; }

        #endregion Properties

        #region Methods
        public void Initialize()
        {

            SongsList = LibraryService.Instance.GetLibrary();
        }

        #endregion Methods

        #region Commmands
        public override bool CanPreviousExecute()
        {
            return true;
        }
        public override void OnPreviousExecute()
        {

        }

        public override bool CanPlayExecute()
        {
            switch (_player.PlaybackState)
            {
                case PlaybackState.Buffering:
                    {
                        return true;
                    }
                case PlaybackState.Playing:
                    {
                        return true;
                    }
                case PlaybackState.Stopped:
                    {
                        return true;
                    }
            }
            return false;
        }

        public async override void OnPlayExecute()
        {
            base.Media = _selectedSong;
            if (!_player.IsMediaLoaded(base.Media))
            {
                await _player.LoadMedia(base.Media).ConfigureAwait(true);
            }

            _player.Play();
            switch (_player.PlaybackState)
            {
                case PlaybackState.Buffering:
                    {
                        _player.Play();
                        break;
                    }
                case PlaybackState.Playing:
                    {
                        _player.Pause();
                        break;
                    }
            }
        }

        public override bool CanNextExecute()
        {
            return true;
        }

        public override void OnNextExecute()
        {

        }

        #endregion Commands

        #region Events
        /// <summary>
        /// PlayerService playback state changed event handler.
        /// </summary>
        /// <param name="sender">The sending object.</param>
        /// <param name="args">Event arguments.</param>
        public void OnPlaybackStateChanged(object sender, PlaybackStateChangedEventArgs args)
        {
            OnPropertyChanged("PlayButtonText");
        }

        #endregion Events
    }
}
